# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-07-02 14:27
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wooey', '0024_script_parameter_m2m_data'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='scriptparameter',
            name='script_version',
        ),
    ]
